package com.caimei365.commodity.components;

import com.aliyun.opensearch.DocumentClient;
import com.aliyun.opensearch.OpenSearchClient;
import com.aliyun.opensearch.SearcherClient;
import com.aliyun.opensearch.sdk.dependencies.com.google.common.collect.Lists;
import com.aliyun.opensearch.sdk.dependencies.org.json.JSONArray;
import com.aliyun.opensearch.sdk.dependencies.org.json.JSONObject;
import com.aliyun.opensearch.sdk.generated.OpenSearch;
import com.aliyun.opensearch.sdk.generated.commons.OpenSearchClientException;
import com.aliyun.opensearch.sdk.generated.commons.OpenSearchException;
import com.aliyun.opensearch.sdk.generated.commons.OpenSearchResult;
import com.aliyun.opensearch.sdk.generated.search.*;
import com.aliyun.opensearch.sdk.generated.search.general.SearchResult;
import com.aliyun.opensearch.search.SearchResultDebug;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * OpenSearch 服务工具类
 *
 * @author : Charles
 * @date : 2021/4/6
 */
@Slf4j
@Service
public class SearchOpenService {

    @Value("${aliyunConfig.searchName}")
    private String appName;
    @Value("${aliyunConfig.accessKey}")
    private String accesskey;
    @Value("${aliyunConfig.secret}")
    private String secret;
    @Value("${aliyunConfig.searchHost}")
    private String host;

    public String getAppName(){
        return this.appName;
    }
    /**
     * 推送操作
     *
     * @param document doc
     * @return result
     * @throws OpenSearchClientException exp
     * @throws OpenSearchException exp
     */
    public String pushDocument(String document, String tableName) throws OpenSearchClientException, OpenSearchException {
        // 创建并构造OpenSearch对象
        OpenSearch openSearch = new OpenSearch(accesskey, secret, host);
        // 创建OpenSearchClient对象，并以OpenSearch对象作为构造参数
        OpenSearchClient serviceClient = new OpenSearchClient(openSearch);
        // 创建DocumentClient对象，并以OpenSearchClient对象作为构造参数
        DocumentClient documentClient = new DocumentClient(serviceClient);
        // 推送 到阿里云开放搜索
        OpenSearchResult osr = documentClient.push(document, appName, tableName);

        return osr.getResult();
    }

    /**
     * 获取查询结果
     *
     * @param queryStr      查询类型及参数串
     * @param searchParams  搜索参数obj
     * @return JSONObject
     * @throws OpenSearchClientException exp
     * @throws OpenSearchException       exp
     */
    public JSONObject getResultJsonObject(String queryStr, SearchParams searchParams) throws OpenSearchClientException, OpenSearchException {
        // 创建并构造OpenSearch对象
        OpenSearch openSearch = new OpenSearch(accesskey, secret, host);
        // 创建OpenSearchClient对象，并以OpenSearch对象作为构造参数
        OpenSearchClient serviceClient = new OpenSearchClient(openSearch);
        // 创建SearcherClient对象，并以OpenSearchClient对象作为构造参数
        SearcherClient searcherClient = new SearcherClient(serviceClient);
        // 执行返回查询结果。用户需按code和message，进行异常情况判断。code对应的错误信息查看——错误码文档。
        SearchResult searchResult = searcherClient.execute(searchParams);
        if (appName.contains("test")) {
            // debug请求地址信息
            SearchResultDebug searched = searcherClient.executeDebug(searchParams);
            // 输出查询请求串信息
            log.info(">>>>>>queryUrl: " + searched.getRequestUrl());
        }
        String jsonStr = searchResult.getResult();
        // 处理查询结果
        JSONObject jsonObj = new JSONObject(jsonStr);
        String status = jsonObj.getString("status");
        String requestId = jsonObj.getString("request_id");
        String tracer = jsonObj.getString("tracer");
        JSONObject result = jsonObj.getJSONObject("result");
        JSONArray errors = jsonObj.getJSONArray("errors");
        log.info(">>>>>>query("+queryStr+"): " + status + ",request_id=" +requestId+ ",errors=" + errors.toString() + ",tracer="+tracer);
        if ("OK".equals(status)) {
            result.put("status", 1);
        } else {
            result.put("status", 0);
        }
        return result;
    }

    /**
     * 获取默认查询参数
     *
     * @param queryStr      查询类型及参数
     * @param fetchFields   返回字段
     * @param summaryFields 摘要字段
     * @param num           页码
     * @param size          每页数量
     * @return searchParams
     */
    public SearchParams getSearchParams(String queryStr, ArrayList<String> fetchFields, List<Map<String, String>> summaryFields, String rankName, int num, int size, String sortField, Integer sortType) {
        // 定义Config对象，用于设定config子句参数，指定应用名，分页，数据返回格式等等
        Config config = new Config(Lists.newArrayList(appName));
        config.setStart(num-1);
        config.setHits(size);
        // 设置搜索结果返回应用中哪些字段
        config.setFetchFields(fetchFields);
        // 设置返回格式为json格式
        config.setSearchFormat(SearchFormat.JSON);
        // 创建参数对象
        SearchParams searchParams = new SearchParams(config);
        // 指定搜索的关键词，这里要指定在哪个索引上搜索
        // 如果不指定的话默认在使用“default”索引（索引字段名称是您在您的数据结构中的“索引字段列表”中对应字段。）
        // 若需多个索引组合查询，需要在setQuery处合并，否则若设置多个setQuery子句，则后面的子句会替换前面子句
        searchParams.setQuery(queryStr);

        // 创建sort对象，并设置二维排序
        Sort sorter = new Sort();
        if (StringUtils.isNotEmpty(sortField)) {
            Order order = (1 == sortType) ? Order.DECREASE : Order.INCREASE;
            // 设置排序字段
            sorter.addToSortFields(new SortField(sortField, order));
        }
        // 以RANK相关性算分降序
        sorter.addToSortFields(new SortField("RANK", Order.DECREASE));
        //添加Sort对象参数
        searchParams.setSort(sorter);
        if (queryStr.contains(rankName + ":")) {
            // 设置粗精排表达式
            Rank rank = new Rank();
            rank.setFirstRankName("default");
            rank.setSecondRankName(rankName);
            searchParams.setRank(rank);
        }
        if (null != summaryFields && !summaryFields.isEmpty()) {
            for (Map<String, String> summaryField : summaryFields) {
                String field = summaryField.get("field");
                Integer length = Integer.valueOf(summaryField.get("length"));
                // 设置搜索结果摘要信息
                Summary summary = new Summary();
                // 指定的生效的字段。此字段必需为可分词的text类型的字段。
                summary.setSummary_field(field);
                // 片段长度
                summary.setSummary_len(length.toString());
                // 飘红标签
                summary.setSummary_element("em");
                // 片段链接符
                summary.setSummary_ellipsis("...");
                // 片段数量
                summary.setSummary_snippet("1");
                // 添加Summary对象参数
                searchParams.addToSummaries(summary);
            }
        }
        return searchParams;
    }

    /**
     * 获取默认滚动查询参数
     *
     * @param queryStr      查询类型及参数
     * @param fetchFields   返回字段
     * @param size          每页数量
     * @return searchParams
     */
    public SearchParams getScrollParams(String queryStr, ArrayList<String> fetchFields, int size, String sortField, Integer sortType) {
        // 定义Config对象，用于设定config子句参数，指定应用名，分页，数据返回格式等等
        Config config = new Config(Lists.newArrayList(appName));
        // config.setStart(0) scroll该参数不起作用，默认为0
        config.setHits(size);
        // 设置搜索结果返回应用中哪些字段
        config.setFetchFields(fetchFields);
        // 设置返回格式为json格式
        config.setSearchFormat(SearchFormat.JSON);
        // 创建参数对象
        SearchParams searchParams = new SearchParams(config);
        // 设置查询子句，若需多个索引组合查询，需要setQuery处合并，否则若设置多个setQuery后面的会替换前面查询
        searchParams.setQuery(queryStr);
        if (StringUtils.isNotEmpty(sortField)) {
            // 创建sort对象，并设置二维排序
            Sort sorter = new Sort();
            Order order = (1 == sortType) ? Order.DECREASE : Order.INCREASE;
            // 设置排序字段
            sorter.addToSortFields(new SortField(sortField, order));
            //添加Sort对象参数
            searchParams.setSort(sorter);
        }
        return searchParams;
    }

    /**
     * 获取查询商品的参数
     *
     * @param queryStr  查询类型及参数
     * @param size      每页数量
     * @param sortField 排序字段
     * @param sortType  升降序
     * @return searchParams
     */
    public SearchParams getProductParams(String queryStr, String filter, Integer identity, int num, int size, String sortField, Integer sortType) {
        // 定义Config对象，用于设定config子句参数，指定应用名，分页，数据返回格式等等
        Config config = new Config(Lists.newArrayList(appName));
        config.setStart(num-1);
        config.setHits(size);
        // 设置搜索结果返回应用中哪些字段
        config.setFetchFields(Lists.newArrayList("id", "p_id", "p_name", "p_image", "p_brand_name", "p_unit", "p_code", "p_price_flag", "p_price_grade", "p_supplier_id", "p_keyword", "p_act_flag"));
        // 注意：config子句中的rerank_size参数，在Rank类对象中设置
        // 设置返回格式为json格式
        config.setSearchFormat(SearchFormat.JSON);

        // 创建参数对象
        SearchParams searchParams = new SearchParams(config);
        // 指定搜索的关键词，这里要指定在哪个索引上搜索
        // 如果不指定的话默认在使用“default”索引（索引字段名称是您在您的数据结构中的“索引字段列表”中对应字段。）
        // 若需多个索引组合查询，需要在setQuery处合并，否则若设置多个setQuery子句，则后面的子句会替换前面子句
        searchParams.setQuery(queryStr);

        String thisFilter = StringUtils.isNotEmpty(filter) ? (filter + " AND ") : "";
        // identity: 0个人，1协销，2会员机构，3供应商，4普通机构
        // p_valid：0逻辑删除 1待审核 2已上架 3已下架 8审核未通过 9已隐身 10已冻结
        // p_visibility：3:所有人可见，2:普通机构可见，1:会员机构可见
        if (identity == 1) {
            // 协销 | 综合供应商
            thisFilter += "(p_valid=2 OR p_valid=3 OR p_valid=9) AND p_type=1";
        } else if (identity == 2) {
            // 会员机构
            thisFilter += "p_valid=2 AND p_type=1";
        } else if (identity == 4) {
            // 普通机构
            thisFilter += "(p_visibility=2 OR p_visibility=3) AND p_valid=2 AND p_type=1";
        } else {
            // 游客|所有人
            thisFilter += "p_visibility=3 AND p_valid=2 AND p_type=1";
        }
        searchParams.setFilter(thisFilter);

        // 设置聚合打散子句
        Distinct dist = new Distinct();
        // 设置dist_key
        dist.setKey("p_id");
        // 设置dist_count
        dist.setDistCount(1);
        // 设置dist_times
        dist.setDistTimes(1);
        // 设置reserved
        dist.setReserved(false);
        // 设置update_total_hit
        dist.setUpdateTotalHit(false);
        // 设置过滤条件
        dist.setDistFilter("p_id>0");
        // 设置grade
        dist.setGrade("1.2");
        searchParams.addToDistincts(dist);

        if (queryStr.contains("product:")) {
            // 设置粗精排表达式
            Rank rank = new Rank();
            rank.setFirstRankName("default");
            rank.setSecondRankName("product");
            searchParams.setRank(rank);
        }
        // 创建sort对象，并设置二维排序
        Sort sorter = new Sort();
        // 价格，销量，人气 排序
        String[] sortFields = {"price", "sales", "favorite"};
        if (StringUtils.isNotEmpty(sortField) && Arrays.asList(sortFields).contains(sortField)) {
            Order order = (1 == sortType) ? Order.DECREASE : Order.INCREASE;
            // 设置排序字段
            sorter.addToSortFields(new SortField("p_"+sortField, order));
        }
        // 以RANK相关性算分降序
        sorter.addToSortFields(new SortField("RANK", Order.DECREASE));
        // 默认商品ID倒序
        sorter.addToSortFields(new SortField("p_id", Order.DECREASE));
        //添加Sort对象参数
        searchParams.setSort(sorter);
        // 设置搜索结果摘要信息
        Summary summary = new Summary();
        // 指定的生效的字段。此字段必需为可分词的text类型的字段。
        summary.setSummary_field("p_name");
        // 片段长度
        summary.setSummary_len("120");
        // 飘红标签
        summary.setSummary_element("em");
        // 片段链接符
        summary.setSummary_ellipsis("...");
        // 片段数量
        summary.setSummary_snippet("1");
        // 添加Summary对象参数
        searchParams.addToSummaries(summary);

        return searchParams;
    }

    /**
     * 获取滚动查询商品的参数
     *
     * @param queryStr  查询类型及参数
     * @param size      每页数量
     * @param sortField 排序字段
     * @param sortType  升降序
     * @return searchParams
     */
    public SearchParams getScrollProductParams(String queryStr, Integer identity, int size, String sortField, Integer sortType) {
        // 定义Config对象，用于设定config子句参数，指定应用名，分页，数据返回格式等等
        Config config = new Config(Lists.newArrayList(appName));
        // config.setStart(0) scroll该参数不起作用，默认为0
        config.setHits(size);
        // 设置搜索结果返回应用中哪些字段
        config.setFetchFields(Lists.newArrayList("id", "p_id", "p_name", "p_image", "p_brand_name", "p_unit", "p_code", "p_price_flag", "p_price_grade", "p_supplier_id", "p_keyword", "p_act_flag"));
        // 设置返回格式为json格式
        config.setSearchFormat(SearchFormat.JSON);
        // 创建参数对象
        SearchParams searchParams = new SearchParams(config);
        // 设置查询子句，若需多个索引组合查询，需要setQuery处合并，否则若设置多个setQuery后面的会替换前面查询
        searchParams.setQuery(queryStr);

        // identity: 0个人，1协销，2会员机构，3供应商，4普通机构
        // p_valid：0逻辑删除 1待审核 2已上架 3已下架 8审核未通过 9已隐身 10已冻结
        // p_visibility：3:所有人可见，2:普通机构可见，1:会员机构可见
        String thisFilter = "";
        if (identity == 1) {
            // 协销 | 综合供应商
            thisFilter += "(p_valid=2 OR p_valid=3 OR p_valid=9) AND p_type=1";
        } else if (identity == 2) {
            // 会员机构
            thisFilter += "p_valid=2 AND p_type=1";
        } else if (identity == 4) {
            // 普通机构
            thisFilter += "(p_visibility=2 OR p_visibility=3) AND p_valid=2 AND p_type=1";
        } else {
            // 游客|所有人
            thisFilter += "p_visibility=3 AND p_valid=2 AND p_type=1";
        }
        searchParams.setFilter(thisFilter);

        // 创建sort对象，并设置二维排序
        Sort sorter = new Sort();
        // 价格，销量，人气 排序
        String[] sortFields = {"p_price", "p_sales", "p_favorite"};
        if (StringUtils.isNotEmpty(sortField) && Arrays.asList(sortFields).contains(sortField)) {
            Order order = (1 == sortType) ? Order.DECREASE : Order.INCREASE;
            // 设置排序字段
            sorter.addToSortFields(new SortField(sortField, order));
        }else{
            // 默认商品ID倒序
            sorter.addToSortFields(new SortField("p_id", Order.DECREASE));
        }
        //添加Sort对象参数
        searchParams.setSort(sorter);

        return searchParams;
    }

    /**
     * 滚动查询
     *
     * @param searchParams  查询参数对象
     * @param queryStr 查询类型及参数串
     * @param pageNum       页码
     * @param pageSize      每页数量
     * @param requestPageSize  每次scroll数量
     * @return list
     */
    public JSONObject deepPagingForParams(SearchParams searchParams, String queryStr, int maxSearchNum, int requestPageSize, int pageNum, int pageSize) throws OpenSearchClientException, OpenSearchException {
        //设置scroll方式查询
        DeepPaging deep =new DeepPaging();
        //不设置默认为1m表示1分钟，该参数表示本次返回的scrollid的有效期
        deep.setScrollExpire("3m");
        //添加DeepPaging对象参数
        searchParams.setDeepPaging(deep);
        // 查询推送
        JSONObject pageObj = new JSONObject();
        JSONObject result = getResultJsonObject(queryStr, searchParams);
        int status = result.getInt("status");
        pageObj.put("status", status);
        if (1 == status) {
            int requestSize = pageNum*pageSize;
            int total = result.getInt("total");
            int totalPage = (int) Math.ceil( (double)total/pageSize);
            JSONArray pageData = new JSONArray();
            int loop = (int)Math.ceil((double) requestSize / requestPageSize);
            for (int l=1; l<=loop; l++) {
                //第一次执行不返回数据，主要返回scroll_id值，第二次调用查询时设置scroll_id
                deep.setScrollId(result.getString("scroll_id"));
                //不设置默认为1m表示1分钟，如不想使用默认值，每次重新调用前必须要重新设置下
                deep.setScrollExpire("3m");
                // 查询推送
                result = getResultJsonObject(queryStr, searchParams);
                if (1 == status) {
                    JSONArray resultArr = result.getJSONArray("items");
                    if (resultArr.length()>0){
                        if (requestSize > maxSearchNum){
                            if (l==loop) {
                                int baseSize = Math.min(requestSize - (requestPageSize*(loop-1)), resultArr.length());
                                // 上一页数量往前总数量 - 上一轮循环后总数量 = 当前循环的上一页之前数量
                                int baseNum = pageSize*(pageNum-1) - requestPageSize*(loop-1);
                                for (int i=0; i<pageSize; i++) {
                                    int index = baseNum + i;
                                    if (index<baseSize){
                                        JSONObject item = resultArr.getJSONObject(index);
                                        pageData.put(i, item);
                                    } else {
                                        break;
                                    }
                                }
                            }
                        } else {
                            int baseNum = (pageNum-1)*pageSize;
                            int currentTotal = requestSize;
                            if (totalPage == pageNum){
                                currentTotal = total;
                            }
                            for (int i=0; i<pageSize; i++) {
                                int index = baseNum + i;
                                if (index<currentTotal){
                                    JSONObject item = resultArr.getJSONObject(index);
                                    pageData.put(i, item);
                                } else {
                                    break;
                                }
                            }
                        }
                    }
                }
            }
            pageObj.put("total", total);
            pageObj.put("items", pageData);
            log.info(">>>>>>pageResult("+queryStr+"): pageSize(" + pageSize +"),pageNum("+pageNum+"),total("+total+")");
        }
        return pageObj;
    }




}
