package com.caimei365.commodity.controller;

import com.caimei365.commodity.model.ResponseJson;
import com.caimei365.commodity.model.dto.UpdateIndexDto;
import com.caimei365.commodity.service.SearchIndexService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * 更新搜索索引（阿里云搜索）
 *
 * @author : Charles
 * @date : 2021/4/6
 */
@Api(tags="搜索索引更新API")
@RestController
@RequiredArgsConstructor
@RequestMapping("/commodity/search/index")
public class SearchIndexApi {

    private final SearchIndexService searchIndexService;

    /**
     * 初始化（添加）所有索引
     *
     * spi旧接口：/search/manage/init/all
     * @return totalRecord
     */
    @ApiOperation("初始化所有索引")
    @PostMapping("/init")
    public ResponseJson<Integer> initAllIndexes(){
        return searchIndexService.initAllIndexes();
    }

    /**
     * 更新所有商品
     *
     * spi旧接口：/search/manage/update/product/all
     * @return productRecord
     */
    @ApiOperation("更新所有商品")
    @PostMapping("/product/all")
    public ResponseJson<Integer> updateAllProductIndexes(){
        return searchIndexService.updateAllProductIndexes();
    }


    /**
     * 更新所有供应商
     *
     * spi旧接口：/search/manage/update/supplier/all
     * @return shopRecord
     */
    @ApiOperation("更新所有供应商")
    @PostMapping("/shop/all")
    public ResponseJson<Integer> updateAllSupplierIndexes(){
        return searchIndexService.updateAllSupplierIndexes();
    }

    /**
     * 更新所有项目仪器
     *
     * spi旧接口：/search/manage/update/equipment/all
     * @return equipmentRecord
     */
    @ApiOperation("更新所有项目仪器")
    @PostMapping("/equipment/all")
    public ResponseJson<Integer> updateAllEquipmentIndexes(){
        return searchIndexService.updateAllEquipmentIndexes();
    }

    /**
     * 更新所有文章
     *
     * spi旧接口：/search/manage/update/article/all
     * @return articleRecord
     */
    @ApiOperation("更新所有文章")
    @PostMapping("/article/all")
    public ResponseJson<Integer> updateAllArticleIndexes(){
        return searchIndexService.updateAllArticleIndexes();
    }

    /**
     * 根据商品Id更新
     *
     * spi旧接口：/search/manage/update/product
     *
     * @param updateIndexDto {
     *                          id           主键id
     *                          productId    商品Id
     *                          shopId       供应商Id
     *                          equipmentId  项目仪器id
     *                          articleId    文章id
     *                    }
     * @return int
     */
    @ApiOperation("根据商品Id更新")
    @PostMapping("/update/product")
    public ResponseJson<Integer> updateProductIndex(UpdateIndexDto updateIndexDto){
        return searchIndexService.updateProductIndexById(updateIndexDto.getProductId());
    }

    /**
     * 根据供应商Id更新
     *
     * spi旧接口：/search/manage/update/supplier
     * @param updateIndexDto {
     *                          id           主键id
     *                          productId    商品Id
     *                          shopId       供应商Id
     *                          equipmentId  项目仪器id
     *                          articleId    文章id
     *                    }
     * @return int
     */
    @ApiOperation("根据供应商Id更新")
    @PostMapping("/update/shop")
    public ResponseJson<Integer> updateSupplierIndex(UpdateIndexDto updateIndexDto){
        return searchIndexService.updateSupplierIndexById(updateIndexDto.getShopId());
    }

    /**
     * 根据项目仪器Id更新
     *
     * spi旧接口：/search/manage/update/equipment
     * @param updateIndexDto {
     *                          id           主键id
     *                          productId    商品Id
     *                          shopId       供应商Id
     *                          equipmentId  项目仪器id
     *                          articleId    文章id
     *                    }
     * @return int
     */
    @ApiOperation("根据仪器Id更新")
    @PostMapping("/update/equipment")
    public ResponseJson<Integer> updateEquipmentIndex(UpdateIndexDto updateIndexDto){
        return searchIndexService.updateEquipmentIndexById(updateIndexDto.getEquipmentId());
    }

    /**
     * 根据文章Id更新
     *
     * spi旧接口：/search/manage/update/article
     * @param updateIndexDto {
     *                          id           主键id
     *                          productId    商品Id
     *                          shopId       供应商Id
     *                          equipmentId  项目仪器id
     *                          articleId    文章id
     *                    }
     * @return int
     */
    @ApiOperation("根据文章Id更新")
    @PostMapping("/update/article")
    public ResponseJson<Integer> updateArticleIndex(UpdateIndexDto updateIndexDto){
        return searchIndexService.updateArticleIndexById(updateIndexDto.getArticleId());
    }

    /**
     * 根据商品Id删除
     *
     * spi旧接口：/search/manage/delete/product
     * @param updateIndexDto {
     *                          id           主键id
     *                          productId    商品Id
     *                          shopId       供应商Id
     *                          equipmentId  项目仪器id
     *                          articleId    文章id
     *                    }
     */
    @ApiOperation("根据商品Id删除")
    @PostMapping("/delete/product")
    public ResponseJson<Integer> deleteProductIndex(UpdateIndexDto updateIndexDto){
        return searchIndexService.deleteProductIndexById(updateIndexDto.getProductId());
    }

    /**
     * 根据供应商Id删除
     *
     * spi旧接口：/search/manage/delete/supplier
     * @param updateIndexDto {
     *                          id           主键id
     *                          productId    商品Id
     *                          shopId       供应商Id
     *                          equipmentId  项目仪器id
     *                          articleId    文章id
     *                    }
     */
    @ApiOperation("根据供应商Id删除")
    @PostMapping("/delete/shop")
    public ResponseJson<Integer> deleteSupplierIndex(UpdateIndexDto updateIndexDto){
        return searchIndexService.deleteSupplierIndexById(updateIndexDto.getShopId());
    }

    /**
     * 根据项目仪器Id删除
     *
     * spi旧接口：/search/manage/delete/equipment
     * @param updateIndexDto {
     *                          id           主键id
     *                          productId    商品Id
     *                          shopId       供应商Id
     *                          equipmentId  项目仪器id
     *                          articleId    文章id
     *                    }
     */
    @ApiOperation("根据项目仪器Id删除")
    @PostMapping("/delete/equipment")
    public ResponseJson<Integer> deleteEquipmentIndex(UpdateIndexDto updateIndexDto){
        return searchIndexService.deleteEquipmentIndexById(updateIndexDto.getEquipmentId());
    }

    /**
     * 根据文章Id删除
     *
     * spi旧接口：/search/manage/delete/article
     * @param updateIndexDto {
     *                          id           主键id
     *                          productId    商品Id
     *                          shopId       供应商Id
     *                          equipmentId  项目仪器id
     *                          articleId    文章id
     *                    }
     */
    @ApiOperation("根据文章Id删除")
    @PostMapping("/delete/article")
    public ResponseJson<Integer> deleteArticleIndex(UpdateIndexDto updateIndexDto){
        return searchIndexService.deleteArticleIndexById(updateIndexDto.getArticleId());
    }

    /**
     * 根据主文档Id删除
     *
     * spi旧接口：/search/manage/delete/main
     * @param updateIndexDto {
     *                          id           主键id
     *                          productId    商品Id
     *                          shopId       供应商Id
     *                          equipmentId  项目仪器id
     *                          articleId    文章id
     *                    }
     */
    @ApiOperation("根据主文档Id删除")
    @PostMapping("/delete/main")
    public ResponseJson<Integer> deleteMainIndex(UpdateIndexDto updateIndexDto){
        return searchIndexService.deleteMainIndexById(updateIndexDto.getId());
    }
}
