package com.caimei365.commodity.controller;

import com.caimei365.commodity.model.ResponseJson;
import com.caimei365.commodity.service.SearchProductService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

/**
 * 搜索商品（阿里云搜索）
 *
 * @author : Charles
 * @date : 2021/4/7
 */
@Api(tags="搜索商品API")
@RestController
@RequiredArgsConstructor
@RequestMapping("/commodity/search/query")
public class SearchProductApi {

    private final SearchProductService searchProductService;

    /**
     * 根据关键词搜索商品
     *
     * spi旧接口：/search/query/product
     *
     * @param keyword 搜索关键字
     * @return JsonStr(list)
     */
    @ApiOperation("根据keyword搜索商品")
    @ApiImplicitParams({
        @ApiImplicitParam(required = false, name = "keyword", value = "搜索关键字"),
        @ApiImplicitParam(required = false, name = "identity", value = "用户身份: 0个人，1协销，2会员机构，3供应商，4普通机构"),
        @ApiImplicitParam(required = false, name = "pageNum", value = "页码"),
        @ApiImplicitParam(required = false, name = "pageSize", value = "每页数量"),
        @ApiImplicitParam(required = false, name = "sortField", value = "排序字段：价格price，销量sales，人气favorite"),
        @ApiImplicitParam(required = false, name = "sortType", value = "排序规则：1降序，其他升序")
    })
    @GetMapping("product")
    public ResponseJson<String> queryProductByKeyword(String keyword,
                                                      @RequestParam(value = "identity", defaultValue = "0") Integer identity,
                                                      @RequestParam(value = "pageNum", defaultValue = "1") int pageNum,
                                                      @RequestParam(value = "pageSize", defaultValue = "20") int pageSize,
                                                      String sortField, Integer sortType) {
        return searchProductService.queryProductByKeyword(keyword, identity, pageNum, pageSize, sortField, sortType);
    }

    /**
     * 根据分类Id搜索商品
     *
     * spi旧接口：/search/query/product/type
     *
     * @param id     bigTypeID/smallTypeID/tinyTypeID
     * @param idType id类型：1一级分类，2二级分类，3三级分类
     */
    @ApiOperation("根据分类Id搜索商品")
    @ApiImplicitParams({
        @ApiImplicitParam(required = true, name = "id", value = "bigTypeID/smallTypeID/tinyTypeID"),
        @ApiImplicitParam(required = true, name = "idType", value = "id类型：1一级分类，2二级分类，3三级分类"),
        @ApiImplicitParam(required = false, name = "identity", value = "用户身份: 0个人，1协销，2会员机构，3供应商，4普通机构"),
        @ApiImplicitParam(required = false, name = "pageNum", value = "页码"),
        @ApiImplicitParam(required = false, name = "pageSize", value = "每页数量"),
        @ApiImplicitParam(required = false, name = "sortField", value = "排序字段：价格price，销量sales，人气favorite"),
        @ApiImplicitParam(required = false, name = "sortType", value = "排序规则：1降序，其他升序")
    })
    @GetMapping("product/type")
    public ResponseJson<String> queryProductByType(Integer id, Integer idType,
                                            @RequestParam(value = "identity", defaultValue = "0") Integer identity,
                                            @RequestParam(value = "pageNum", defaultValue = "1") int pageNum,
                                            @RequestParam(value = "pageSize", defaultValue = "20") int pageSize,
                                            String sortField, Integer sortType) {
        return searchProductService.queryProductByType(id, idType, identity, pageNum, pageSize, sortField, sortType);
    }

    /**
     * 根据小程序运营分类Id搜索商品
     *
     * spi旧接口：/search/query/product/classify
     *
     * @param id        运营级分类Id
     * @param identity  用户身份 1:协销， 2:会员机构
     */
    @ApiOperation("根据小程序运营分类Id搜索商品")
    @ApiImplicitParams({
        @ApiImplicitParam(required = true, name = "id", value = "运营级分类Id"),
        @ApiImplicitParam(required = false, name = "identity", value = "用户身份: 0个人，1协销，2会员机构，3供应商，4普通机构"),
        @ApiImplicitParam(required = false, name = "pageNum", value = "页码"),
        @ApiImplicitParam(required = false, name = "pageSize", value = "每页数量"),
        @ApiImplicitParam(required = false, name = "sortField", value = "排序字段：价格price，销量sales，人气favorite"),
        @ApiImplicitParam(required = false, name = "sortType", value = "排序规则：1降序，其他升序")
    })
    @GetMapping("product/classify")
    public ResponseJson<String> queryProductByClassify(Integer id,
                                            @RequestParam(value = "identity", defaultValue = "0") Integer identity,
                                            @RequestParam(value = "pageNum", defaultValue = "1") int pageNum,
                                            @RequestParam(value = "pageSize", defaultValue = "20") int pageSize,
                                            String sortField, Integer sortType) {
        return searchProductService.queryProductByClassify(id, identity, pageNum, pageSize, sortField, sortType);
    }

    /**
     * 根据供应商Id搜索商品（商铺内搜索）
     *
     * spi旧接口：/search/query/product/supplier
     *
     * @param id       供应商Id
     * @param keyword  搜索关键字
     */
    @ApiOperation("根据供应商Id搜索商品（商铺内搜索）")
    @ApiImplicitParams({
        @ApiImplicitParam(required = true, name = "id", value = "供应商Id"),
        @ApiImplicitParam(required = false, name = "keyword", value = "搜索关键字"),
        @ApiImplicitParam(required = false, name = "identity", value = "用户身份: 0个人，1协销，2会员机构，3供应商，4普通机构"),
        @ApiImplicitParam(required = false, name = "pageNum", value = "页码"),
        @ApiImplicitParam(required = false, name = "pageSize", value = "每页数量"),
        @ApiImplicitParam(required = false, name = "sortField", value = "排序字段：价格price，销量sales，人气favorite"),
        @ApiImplicitParam(required = false, name = "sortType", value = "排序规则：1降序，其他升序")
    })
    @GetMapping("product/shop")
    public ResponseJson<String> querySupplierProduct(Integer id, String keyword,
                                           @RequestParam(value = "identity", defaultValue = "0") Integer identity,
                                           @RequestParam(value = "pageNum", defaultValue = "1") int pageNum,
                                           @RequestParam(value = "pageSize", defaultValue = "20") int pageSize,
                                           String sortField, Integer sortType) {
        return searchProductService.querySupplierProduct(id, keyword, identity, pageNum, pageSize, sortField, sortType);
    }

}
