package com.caimei365.commodity.controller;

import com.caimei365.commodity.model.ResponseJson;
import com.caimei365.commodity.service.SearchQueryService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.web.bind.annotation.*;

import java.util.Map;

/**
 * 搜索查询（阿里云搜索）
 *
 * @author : Charles
 * @date : 2021/4/6
 */
@Api(tags="搜索查询API")
@RestController
@RequiredArgsConstructor
@RequestMapping("/commodity/search/query")
public class SearchQueryApi {

    private final SearchQueryService searchQueryService;

    /**
     * 根据文档Id获取阿里云索引Id
     *
     * spi旧接口：/search/query/(id,pid,sid,eid,aid)
     * @return int
     */
    @ApiOperation("根据文档Id获取阿里云索引Id")
    @ApiImplicitParams({
        @ApiImplicitParam(required = false, name = "docType", value = "文档类型（product,shop,equipment,article）"),
        @ApiImplicitParam(required = false, name = "docId", value = "文档Id")
    })
    @GetMapping("id")
    public Integer getIdByDocId(String docType, Integer docId) {
        return searchQueryService.getIdByDocId(docType, docId);
    }

    /**
     * 根据文档类型获取阿里云索引Id集合
     *
     * spi旧接口：/search/query/(product,supplier,equipment,article)/ids
     * @return map
     */
    @ApiOperation("根据文档类型获取阿里云索引Id集合")
    @ApiImplicitParams({
        @ApiImplicitParam(required = true, name = "docType", value = "文档类型（product,shop,equipment,article）"),
        @ApiImplicitParam(required = false, name = "pageNum", value = "页码"),
        @ApiImplicitParam(required = false, name = "pageSize", value = "每页数量")
    })
    @GetMapping("ids")
    public Map<Integer, Integer> getMainIdsByDocType(String docType,
                                                     @RequestParam(value = "pageNum", defaultValue = "1") int pageNum,
                                                     @RequestParam(value = "pageSize", defaultValue = "20") int pageSize) {
        return searchQueryService.getMainIdsByDocType(docType, pageNum, pageSize);
    }

    /**
     * 根据文档类型获取数量
     *
     * spi旧接口：/search/query/record
     * @return int
     */
    @ApiOperation("根据文档类型获取数量")
    @ApiImplicitParam(required = true, name = "docType", value = "文档类型（product,shop,equipment,article）")
    @GetMapping("record")
    public Integer getRecordByDocType(String docType) {
        return searchQueryService.getRecordByDocType(docType);
    }

    /**
     * 根据文档类型获取数量(去重用)
     *
     * spi旧接口：/search/query/acount
     * @return int
     */
    @ApiOperation("根据文档Id获取数量(去重用)")
    @ApiImplicitParams({
        @ApiImplicitParam(required = false, name = "docType", value = "文档类型（product,shop,equipment,article）"),
        @ApiImplicitParam(required = false, name = "docId", value = "文档Id")
    })
    @GetMapping("count")
    public Integer getCountByDocId(String docType, Integer docId) {
        return searchQueryService.getCountByDocId(docType, docId);
    }

    /**
     * 根据关键词搜索供应商
     *
     * spi旧接口：/search/query/supplier
     *
     * @param keyword 搜索关键字
     */
    @ApiOperation("根据关键词搜索供应商")
    @ApiImplicitParams({
        @ApiImplicitParam(required = false, name = "keyword", value = "搜索关键字"),
        @ApiImplicitParam(required = false, name = "identity", value = "用户身份: 0个人，1协销，2会员机构，3供应商，4普通机构"),
        @ApiImplicitParam(required = false, name = "pageNum", value = "页码"),
        @ApiImplicitParam(required = false, name = "pageSize", value = "每页数量")
    })
    @GetMapping("shop")
    public ResponseJson<String> querySupplierByKeyword(String keyword,
                                                       @RequestParam(value = "identity", defaultValue = "0") Integer identity,
                                                       @RequestParam(value = "pageNum", defaultValue = "1") int pageNum,
                                                       @RequestParam(value = "pageSize", defaultValue = "10") int pageSize) {
        return searchQueryService.querySupplierByKeyword(keyword, identity, pageNum, pageSize);
    }

    /**
     * 根据关键词搜索项目仪器
     *
     * spi旧接口：/search/query/equipment
     *
     * @param keyword 搜索关键字
     */
    @ApiOperation("根据关键词搜索项目仪器")
    @ApiImplicitParams({
        @ApiImplicitParam(required = false, name = "keyword", value = "搜索关键字"),
        @ApiImplicitParam(required = false, name = "pageNum", value = "页码"),
        @ApiImplicitParam(required = false, name = "pageSize", value = "每页数量")
    })
    @GetMapping("equipment")
    public ResponseJson<String> queryEquipmentByKeyword(String keyword,
                                             @RequestParam(value = "pageNum", defaultValue = "1") int pageNum,
                                             @RequestParam(value = "pageSize", defaultValue = "20") int pageSize) {
        return searchQueryService.queryEquipmentByKeyword(keyword, pageNum, pageSize);
    }

    /**
     * 根据关键词搜索文章
     *
     * spi旧接口：/search/query/article
     *
     * @param keyword 搜索关键字
     */
    @ApiOperation("根据关键词搜索文章")
    @ApiImplicitParams({
        @ApiImplicitParam(required = false, name = "keyword", value = "搜索关键字"),
        @ApiImplicitParam(required = false, name = "pageNum", value = "页码"),
        @ApiImplicitParam(required = false, name = "pageSize", value = "每页数量")
    })
    @GetMapping("article")
    public ResponseJson<String> queryArticleByKeyword(String keyword,
                                           @RequestParam(value = "pageNum", defaultValue = "1") int pageNum,
                                           @RequestParam(value = "pageSize", defaultValue = "20") int pageSize) {
        return searchQueryService.queryArticleByKeyword(keyword, pageNum, pageSize);
    }

    /**
     * 根据文章ID搜索文章
     *
     * spi旧接口：/search/query/article/{id}
     *
     * @param id 文章id
     */
    @ApiOperation("根据文章ID搜索文章")
    @GetMapping("article/{id}")
    public ResponseJson<String> queryArticleByArticleId(@PathVariable("id") Integer id) {
        return searchQueryService.queryArticleByArticleId(id);
    }

    /**
     * 根据分类搜索文章
     *
     * spi旧接口：/search/query/article/type
     *
     * @param id 分类id
     * @return list
     */
    @ApiOperation("根据分类搜索文章")
    @ApiImplicitParams({
        @ApiImplicitParam(required = false, name = "id", value = "分类id"),
        @ApiImplicitParam(required = false, name = "pageNum", value = "页码"),
        @ApiImplicitParam(required = false, name = "pageSize", value = "每页数量")
    })
    @GetMapping("article/type")
    public ResponseJson<String> queryArticleByType(Integer id,
                                        @RequestParam(value = "pageNum", defaultValue = "1") int pageNum,
                                        @RequestParam(value = "pageSize", defaultValue = "20") int pageSize) {
        return searchQueryService.queryArticleByType(id, pageNum, pageSize);
    }

    /**
     * 根据标签搜索文章
     *
     * spi旧接口：/search/query/article/label
     *
     * @param id 标签id
     * @return list
     */
    @ApiOperation("根据标签搜索文章")
    @ApiImplicitParams({
        @ApiImplicitParam(required = false, name = "id", value = "标签id"),
        @ApiImplicitParam(required = false, name = "pageNum", value = "页码"),
        @ApiImplicitParam(required = false, name = "pageSize", value = "每页数量")
    })
    @GetMapping("article/label")
    public ResponseJson<String> queryArticleByLabel(Integer id,
                                         @RequestParam(value = "pageNum", defaultValue = "1") int pageNum,
                                         @RequestParam(value = "pageSize", defaultValue = "20") int pageSize) {
        return searchQueryService.queryArticleByLabel(id, pageNum, pageSize);
    }



}
