package com.caimei365.commodity.service.impl;

import com.caimei365.commodity.components.PriceUtilService;
import com.caimei365.commodity.mapper.CouponMapper;
import com.caimei365.commodity.mapper.PageMapper;
import com.caimei365.commodity.mapper.ShopMapper;
import com.caimei365.commodity.model.ResponseJson;
import com.caimei365.commodity.model.dto.CollarCouponsDto;
import com.caimei365.commodity.model.dto.RedeemCouponsDto;
import com.caimei365.commodity.model.po.CouponClubPo;
import com.caimei365.commodity.model.po.CouponRedemptionCodePo;
import com.caimei365.commodity.model.vo.ActivityCouponVo;
import com.caimei365.commodity.model.vo.CouponVo;
import com.caimei365.commodity.model.vo.ProductItemVo;
import com.caimei365.commodity.model.vo.ShopVo;
import com.caimei365.commodity.service.CouponService;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.*;

/**
 * Description
 *
 * @author : plf
 * @date : 2021/8/12
 */
@Service
public class CouponServiceImpl implements CouponService {
    @Resource
    private CouponMapper couponMapper;
    @Resource
    private ShopMapper shopMapper;
    @Resource
    private PageMapper pageMapper;
    @Resource
    private PriceUtilService priceUtilService;

    /**
     * 已领取优惠券中心
     *
     * @param userId   机构用户id
     * @param status   使用状态: 1未使用 2已使用 3已失效
     * @param pageNum  页码
     * @param pageSize 每页数量
     */
    @Override
    public ResponseJson<PageInfo<CouponVo>> couponCenter(Integer userId, Integer status, int pageNum, int pageSize) {
        PageHelper.startPage(pageNum, pageSize);
        List<CouponVo> couponList = couponMapper.findCouponClub(userId, status);
        couponList.forEach(coupon -> {
            setShopName(coupon);
            coupon.setUseStatus(status);
        });
        PageInfo<CouponVo> pageInfo = new PageInfo<>(couponList);
        return ResponseJson.success(pageInfo);
    }

    /**
     * set店铺名称
     *
     * @param coupon 优惠券
     */
    private void setShopName(CouponVo coupon) {
        if (coupon.getCouponType() == 3 && coupon.getShopId() != null) {
            //店铺券
            ShopVo shop = shopMapper.getProductShopById(coupon.getShopId());
            if (shop != null && StringUtils.isNotBlank(shop.getName())) {
                String shopName = shop.getName();
                if (shopName.length() > 7) {
                    shopName = shopName.substring(0, 6) + "...";
                }
                coupon.setShopName(shopName);
            }
        }
    }

    /**
     * 活动券部分商品活动数据
     *
     * @param couponId 活动券id
     * @param userId   机构用户id
     * @param source   来源 : 1 网站 ; 2 小程序
     * @param pageNum  页码
     * @param pageSize 每页数量
     */
    @Override
    public ResponseJson<Map<String, Object>> activityPage(Integer couponId, Integer userId, Integer source, int pageNum, int pageSize) {
        Map<String, Object> map = new HashMap<>(2);
        ActivityCouponVo coupon = couponMapper.findCouponById(couponId);
        if (coupon == null) {
            return ResponseJson.error("活动已失效", null);
        }
        PageHelper.startPage(pageNum, pageSize);
        List<ProductItemVo> productList = couponMapper.findCouponProduct(couponId, source);
        productList.forEach(p -> {
            p.setCouponsLogo(true);
            priceUtilService.setProductDetails(userId, p);
        });
        PageInfo<ProductItemVo> pageInfo = new PageInfo<>(productList);
        map.put("coupon", coupon);
        map.put("pageInfo", pageInfo);
        return ResponseJson.success(map);
    }

    @Override
    public ResponseJson<PageInfo<CouponVo>> collarCouponsList(Integer userId, int pageNum, int pageSize) {
        PageHelper.startPage(pageNum, pageSize);
        List<CouponVo> couponList = couponMapper.findCouponList(userId);
        couponList.forEach(this::setShopName);
        PageInfo<CouponVo> pageInfo = new PageInfo<>(couponList);
        return ResponseJson.success(pageInfo);
    }

    @Override
    public ResponseJson<String> collarCoupons(CollarCouponsDto couponsDto) {
        Integer id = couponMapper.findCouponClubByUserId(couponsDto.getUserId(), couponsDto.getCouponId());
        if (id != null && id > 0) {
            return ResponseJson.error("已经领取过了", null);
        }
        CouponClubPo couponClub = new CouponClubPo();
        BeanUtils.copyProperties(couponsDto, couponClub);
        couponClub.setStatus("1");
        couponClub.setCreateDate(new Date());
        couponClub.setDelFlag("0");
        couponMapper.insertCouponClub(couponClub);
        return ResponseJson.success("领取成功");
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public ResponseJson<CouponVo> redeemCoupons(RedeemCouponsDto redeemCouponsDto) {
        CouponRedemptionCodePo redemptionCode = couponMapper.findCouponRedemptionCode(redeemCouponsDto.getRedemptionCode());
        if (redemptionCode == null) {
            return ResponseJson.error("请输入正确的兑换码", null);
        }
        if (redemptionCode.getStatus() == 2) {
            return ResponseJson.error("兑换码已使用，请更换兑换码进行兑换", null);
        }
        CouponVo coupon = couponMapper.getCoupons(redemptionCode.getCouponId());
        Date date = new Date();
        if (coupon == null || date.compareTo(coupon.getStartDate()) < 0 || date.compareTo(coupon.getEndDate()) > 0) {
            return ResponseJson.error("兑换的优惠券已失效", null);
        }
        if (coupon.getCouponType() == 2 && !redeemCouponsDto.getUserId().equals(coupon.getUserId())) {
            return ResponseJson.error("该优惠券属于其他用户，您不能兑换", null);
        }
        if (coupon.getCouponType() == 4) {
            Date registerTime = couponMapper.findUserRegisterTime(redeemCouponsDto.getUserId());
            if (registerTime == null || registerTime.compareTo(coupon.getStartDate()) < 0) {
                return ResponseJson.error("该优惠券只供新用户使用，您不能兑换", null);
            }
        }
        setShopName(coupon);
        CouponClubPo couponClub = new CouponClubPo();
        couponClub.setCouponId(redemptionCode.getCouponId());
        couponClub.setUserId(redeemCouponsDto.getUserId());
        couponClub.setSource(redeemCouponsDto.getSource());
        couponClub.setStatus("1");
        couponClub.setCreateDate(new Date());
        couponClub.setDelFlag("0");
        couponMapper.insertCouponClub(couponClub);
        couponMapper.updateRedemptionCode(redemptionCode.getId(), couponClub.getId());
        return ResponseJson.success(coupon);
    }

    @Override
    public ResponseJson<Map<String, Integer>> couponsCount(Integer userId) {
        Map<String, Integer> map = new HashMap<>(3);
        List<CouponVo> unusedList = couponMapper.findCouponClub(userId, 1);
        List<CouponVo> usedList = couponMapper.findCouponClub(userId, 2);
        List<CouponVo> expiredList = couponMapper.findCouponClub(userId, 3);
        int unusedNum = unusedList == null ? 0 : unusedList.size();
        int usedNum = usedList == null ? 0 : usedList.size();
        int expiredNum = expiredList == null ? 0 : expiredList.size();
        map.put("unusedNum", unusedNum);
        map.put("usedNum", usedNum);
        map.put("expiredNum", expiredNum);
        return ResponseJson.success(map);
    }

    @Override
    public ResponseJson<Map<String, Object>> detailsCoupons(Integer userId, Integer productId, Integer source, Integer status) {
        Map<String, Object> map = new HashMap<>(3);
        ProductItemVo product = pageMapper.getProductItemById(productId);
        if (product == null) {
            return ResponseJson.error("商品数据异常", null);
        }
        //未领取
        List<CouponVo> notCouponList = couponMapper.findCouponList(userId);
        filterCoupon(source, product, notCouponList);
        map.put("notCouponNum", notCouponList == null ? 0 : notCouponList.size());
        //已领取
        List<CouponVo> couponList = couponMapper.findCouponClub(userId, 1);
        filterCoupon(source, product, couponList);
        map.put("couponNum", couponList == null ? 0 : couponList.size());
        if (status == 1) {
            //未领取
            map.put("couponList", notCouponList);
        } else {
            //已领取
            map.put("couponList", couponList);
        }
        List<CouponVo> list = new ArrayList<>();
        if (notCouponList != null) {
            list.addAll(notCouponList);
        }
        if (couponList != null) {
            list.addAll(couponList);
        }
        list.sort((o1, o2) -> o2.getCouponAmount().compareTo(o1.getCouponAmount()));
        map.put("list", list);
        return ResponseJson.success(map);
    }

    /**
     * 过滤与商品无关的优惠券
     *
     * @param source
     * @param product
     * @param couponList
     */
    private void filterCoupon(Integer source, ProductItemVo product, List<CouponVo> couponList) {
        if (couponList != null && couponList.size() > 0) {
            Iterator<CouponVo> iterator = couponList.iterator();
            while (iterator.hasNext()) {
                CouponVo coupon = iterator.next();
                if (coupon.getCouponType() == 0 && coupon.getProductType() == 2) {
                    //活动券
                    List<Integer> productIds = pageMapper.findAllProductId(coupon.getCouponId(), source);
                    if (!productIds.contains(product.getProductId())) {
                        iterator.remove();
                        continue;
                    }
                }
                if (coupon.getCouponType() == 1 && !coupon.getCategoryType().equals(product.getCommodityType())) {
                    iterator.remove();
                    continue;
                }
                if (coupon.getCouponType() == 3 && !coupon.getShopId().equals(product.getShopId())) {
                    iterator.remove();
                    continue;
                }
                setShopName(coupon);
            }
        }
    }
}
