package com.caimei365.commodity.service.impl;

import com.aliyun.opensearch.sdk.dependencies.org.json.JSONArray;
import com.aliyun.opensearch.sdk.dependencies.org.json.JSONException;
import com.aliyun.opensearch.sdk.dependencies.org.json.JSONObject;
import com.aliyun.opensearch.sdk.generated.commons.OpenSearchClientException;
import com.aliyun.opensearch.sdk.generated.commons.OpenSearchException;
import com.aliyun.opensearch.sdk.generated.search.SearchParams;
import com.caimei365.commodity.components.SearchOpenService;
import com.caimei365.commodity.mapper.SearchMapper;
import com.caimei365.commodity.model.ResponseJson;
import com.caimei365.commodity.model.vo.PaginationVo;
import com.caimei365.commodity.model.search.ProductListVo;
import com.caimei365.commodity.service.SearchProductService;
import com.caimei365.commodity.utils.Json2PojoUtil;
import com.caimei365.commodity.components.PriceUtilService;
import com.github.pagehelper.PageHelper;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;

/**
 * Description
 *
 * @author : Charles
 * @date : 2021/4/7
 */
@Slf4j
@Service
public class SearchProductServiceImpl implements SearchProductService {

    private static final Integer SEARCH_MAX_NUM = 500;
    @Value("${caimei.wwwDomain}")
    private String domain;
    @Resource
    private SearchMapper searchMapper;
    @Resource
    private SearchOpenService searchOpenService;
    @Resource
    private PriceUtilService priceUtilService;
    /**
     * 根据关键词搜索商品
     *
     * @param keyword   搜索关键字
     * @param pageNum   页码
     * @param pageSize  每页数量
     * @param sortField 排序字段
     * @param sortType  升降序0/1
     * @return JsonStr(list)
     */
    @Override
    public ResponseJson<String> queryProductByKeyword(String keyword, Integer identity, int pageNum, int pageSize, String sortField, Integer sortType) {
        String queryStr = "product:'" + keyword + "'";
        if (StringUtils.isEmpty(keyword)) {
            queryStr = "p_all:'1'";
        }
        // 阿里云搜索
        ResponseJson<String> result = queryProduct(queryStr, "", identity, pageNum, pageSize, sortField, sortType);
        if (0 == result.getCode()) {
            return result;
        } else {
            // 阿里云搜索失败，再次从数据库搜索
            return queryProductFromDatabase(pageNum, pageSize, identity, keyword, null, null, null, null, null, sortField, sortType);
        }
    }

    /**
     * 根据分类Id搜索商品
     *
     * @param id        bigTypeID/smallTypeID/tinyTypeID
     * @param idType    id类型：1一级分类，2二级分类，3三级分类
     * @param identity  : 0个人，1协销，2会员机构，3供应商，4普通机构
     * @param pageNum   页码
     * @param pageSize  每页数量
     * @param sortField 排序字段
     * @param sortType  升降序0/1
     * @return JsonStr(list)
     */
    @Override
    public ResponseJson<String> queryProductByType(Integer id, Integer idType, Integer identity, int pageNum, int pageSize, String sortField, Integer sortType) {
        String queryStr = "";
        if (idType == 1) {
            queryStr = "p_bigtype:'" + id + "'";
        } else if (idType == 2) {
            queryStr = "p_smalltype:'" + id + "'";
        } else if (idType == 3) {
            queryStr = "p_tinytype:'" + id + "'";
        } else {
            return ResponseJson.error("idType参数错误", null);
        }
        int requestSize = pageNum*pageSize;
        int requestPageSize = requestSize;
        if (requestSize > SEARCH_MAX_NUM){
            requestPageSize = (int) Math.floor(SEARCH_MAX_NUM/pageSize) * pageSize;
        }
        SearchParams searchParams = searchOpenService.getScrollProductParams(queryStr, identity, requestPageSize, sortField, sortType);
        // 阿里云搜索(滚动查询)
        ResponseJson<String> result = getScrollProductResult(searchParams, queryStr, requestPageSize, pageNum, pageSize);
        if (0 == result.getCode()) {
            return result;
        } else {
            // 阿里云搜索失败，再次从数据库搜索
            if (idType == 1) {
                return queryProductFromDatabase(pageNum, pageSize, identity, null, null, id, null, null, null, sortField, sortType);
            } else if (idType == 2) {
                return queryProductFromDatabase(pageNum, pageSize, identity, null, null, null, id, null, null, sortField, sortType);
            } else {
                return queryProductFromDatabase(pageNum, pageSize, identity, null, null, null, null, id, null, sortField, sortType);
            }
        }
    }

    /**
     * 根据小程序运营分类Id搜索商品
     *
     * @param id        运营级分类Id
     * @param identity  用户身份 1:协销， 2:会员机构
     * @param pageNum   页码
     * @param pageSize  每页数量
     * @param sortField 排序字段
     * @param sortType  升降序0/1
     * @return JsonStr(list)
     */
    @Override
    public ResponseJson<String> queryProductByClassify(Integer id, Integer identity, int pageNum, int pageSize, String sortField, Integer sortType) {
        String queryStr = "p_classify:'" + id + "'";
        int requestSize = pageNum*pageSize;
        int requestPageSize = requestSize;
        if (requestSize > SEARCH_MAX_NUM){
            requestPageSize = (int) Math.floor(SEARCH_MAX_NUM/pageSize) * pageSize;
        }
        SearchParams searchParams = searchOpenService.getScrollProductParams(queryStr, identity, requestPageSize, sortField, sortType);
        // 阿里云搜索(滚动查询)
        ResponseJson<String> result = getScrollProductResult(searchParams, queryStr, requestPageSize, pageNum, pageSize);
        if (0 == result.getCode()) {
            return result;
        } else {
            // 阿里云搜索失败，再次从数据库搜索
            return queryProductFromDatabase(pageNum, pageSize, identity, null, null, null, null, null, id, sortField, sortType);
        }
    }

    /**
     * 根据供应商Id搜索商品（商铺内搜索）
     *
     * @param id        供应商Id
     * @param keyword   搜索关键字
     * @param identity  用户身份 1:协销， 2:会员机构
     * @param pageNum   页码
     * @param pageSize  每页数量
     * @param sortField 排序字段
     * @param sortType  升降序0/1
     * @return JsonStr(list)
     */
    @Override
    public ResponseJson<String> querySupplierProduct(Integer id, String keyword, Integer identity, int pageNum, int pageSize, String sortField, Integer sortType) {
        if (null == id){
            return ResponseJson.error("请提供供应商id" ,null);
        }
        ResponseJson<String> result;
        if (StringUtils.isEmpty(keyword)) {
            String queryStr = "p_supplier:'" + id + "'";
            int requestSize = pageNum*pageSize;
            int requestPageSize = requestSize;
            if (requestSize > SEARCH_MAX_NUM){
                requestPageSize = (int) Math.floor(SEARCH_MAX_NUM/pageSize) * pageSize;
            }
            SearchParams searchParams = searchOpenService.getScrollProductParams(queryStr, identity, requestPageSize, sortField, sortType);
            result = getScrollProductResult(searchParams, queryStr, requestPageSize, pageNum, pageSize);
        } else {
            String queryStr = "product:'" + keyword + "'";
            String filter = "p_supplier_id=" + id;
            result = queryProduct(queryStr, filter, identity, pageNum, pageSize, sortField, sortType);
        }
        if (0 == result.getCode()) {
            return result;
        } else {
            // 阿里云搜索失败，再次从数据库搜索
            return queryProductFromDatabase(pageNum, pageSize, identity, keyword, id, null, null, null, null, sortField, sortType);
        }
    }

    /**
     * 获取数据库商品
     *
     * @param num       页码
     * @param size      每页数量
     * @param identity  用户身份
     * @param keyword   关键词
     * @param shopId    供应商id
     * @param bigTypeId 一级分类id
     * @param smallTypeId 二级分类id
     * @param tinyTypeId 三级分类id
     * @param classifyId 小程序运营分类id
     * @param sortField 排序类型
     * @param sortType  排序顺序
     */
    private ResponseJson<String> queryProductFromDatabase(int num, int size, Integer identity, String keyword, Integer shopId,
                             Integer bigTypeId, Integer smallTypeId, Integer tinyTypeId, Integer classifyId, String sortField, Integer sortType){
        PageHelper.startPage(num, size);
        List<ProductListVo> productList = searchMapper.queryProduct(identity, keyword, shopId, bigTypeId, smallTypeId, tinyTypeId, classifyId, sortField, sortType);
        productList.forEach(product -> {
            product.setPriceGrade(priceUtilService.getPriceGrade(product.getPrice()));
            product.setPrice(0d);
        });
        PaginationVo<ProductListVo> pageData = new PaginationVo<>(productList);
        int total = pageData.getTotalRecord();
        JSONObject pageObj = new JSONObject();
        pageObj.put("total", total);
        pageObj.put("items", productList);
        log.info(">>>>>>数据库容错查询: pageSize(" + size +"),pageNum("+size+"),total("+total+")");
        if (productList.size()>0) {
            return ResponseJson.success(pageObj.toString());
        } else {
            return ResponseJson.error("未查询到文档记录(数据库)", null);
        }
    }






    /**
     * 查询商品（阿里云搜索）
     *
     * @param queryStr  查询类型及参数
     * @param num       页码
     * @param size      每页数量
     * @param sortField 排序字段
     * @param sortType  升降序
     */
    private ResponseJson<String> queryProduct(String queryStr, String filter, Integer identity, int num, int size, String sortField, Integer sortType) {
        SearchParams searchParams;
        int requestSize = num*size;
        if (requestSize > SEARCH_MAX_NUM){
            searchParams = searchOpenService.getProductParams(queryStr, filter, identity,num, size, sortField, sortType);
        } else {
            // 由于阿里云搜索机制问题（分页数据重复），搜索500条以内数据手动分页。
            searchParams = searchOpenService.getProductParams(queryStr, filter, identity,1, requestSize, sortField, sortType);
        }
        try {
            // 查询推送
            JSONObject result = searchOpenService.getResultJsonObject(queryStr, searchParams);
            if (1 == result.getInt("status")) {
                int total = result.getInt("total");
                int totalPage = (int) Math.ceil( (double)total/size);
                JSONArray resultArr = result.getJSONArray("items");
                JSONArray pageData = new JSONArray();
                if (requestSize > SEARCH_MAX_NUM) {
                    pageData = resultArr;
                } else {
                    if (resultArr.length()>0){
                        int baseNum = (num-1)*size;
                        for (int i=0; i<size; i++) {
                            int index = baseNum + i;
                            int currentTotal = requestSize;
                            if (totalPage == num){
                                currentTotal = total;
                            }
                            if (index<currentTotal){
                                JSONObject item = resultArr.getJSONObject(index);
                                pageData.put(i, item);
                            } else {
                                break;
                            }
                        }
                    }
                }
                // 阿里云查询出json转换成商品pojo
                List<ProductListVo> productList = Json2PojoUtil.toProductList(pageData);
                JSONObject pageObj = new JSONObject();
                pageObj.put("total", total);
                pageObj.put("items", productList);
                log.info(">>>>>>pageResult("+queryStr+"): pageSize(" + size +"),pageNum("+num+"),total("+total+")");
                return ResponseJson.success(pageObj.toString());
            } else {
                return ResponseJson.error("未查询到文档记录", null);
            }
        } catch (OpenSearchClientException | OpenSearchException | JSONException e) {
            log.error("查询文档异常：" + e.getMessage());
            return ResponseJson.error("查询文档异常：" + e.getMessage(), null);
        }
    }

    /**
     * 滚动查询
     *
     * @param searchParams  查询参数对象
     * @param queryStr      查询类型及参数串
     * @param requestPageSize  每次scroll数量
     * @param pageNum       页码
     * @param pageSize      每页数量
     * @return list
     */
    private ResponseJson<String> getScrollProductResult(SearchParams searchParams, String queryStr, int requestPageSize, int pageNum, int pageSize) {
        ResponseJson<String> result;
        try {
            JSONObject pageObj = searchOpenService.deepPagingForParams(searchParams, queryStr, SEARCH_MAX_NUM, requestPageSize, pageNum, pageSize);
            if (1 == pageObj.getInt("status")){
                JSONArray pageData = pageObj.getJSONArray("items");
                // 阿里云查询出json转换成商品pojo
                List<ProductListVo> productList = Json2PojoUtil.toProductList(pageData);
                pageObj.put("items", productList);
                result = ResponseJson.success(pageObj.toString());
            } else {
                result = ResponseJson.error("未查询到文档记录", null);
            }
        } catch (OpenSearchClientException | OpenSearchException | JSONException e) {
            log.error("查询文档异常：" + e.getMessage());
            result = ResponseJson.error("查询文档异常：" + e.getMessage(), null);
        }
        return result;
    }
}
