package com.caimei365.commodity.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.caimei365.commodity.components.PriceUtilService;
import com.caimei365.commodity.mapper.PriceMapper;
import com.caimei365.commodity.mapper.PromotionsMapper;
import com.caimei365.commodity.mapper.ShopMapper;
import com.caimei365.commodity.model.ResponseJson;
import com.caimei365.commodity.model.dto.BrandDto;
import com.caimei365.commodity.model.dto.FeaturedDto;
import com.caimei365.commodity.model.dto.ProductDto;
import com.caimei365.commodity.model.po.*;
import com.caimei365.commodity.model.search.ProductListVo;
import com.caimei365.commodity.model.vo.*;
import com.caimei365.commodity.service.ShopService;
import com.caimei365.commodity.utils.ImageUtils;
import com.github.pagehelper.PageHelper;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * Description
 *
 * @author : Charles
 * @date : 2021/4/16
 */
@Slf4j
@Service
public class ShopServiceImpl implements ShopService {
    @Resource
    private ShopMapper shopMapper;
    @Resource
    private PriceMapper priceMapper;
    @Resource
    private PromotionsMapper promotionsMapper;
    @Resource
    private PriceUtilService priceUtilService;
    @Value("${caimei.wwwDomain}")
    private String domain;

    /**
     * 供应商-主推商品
     *
     * @param shopId   供应商Id
     * @param identity 用户身份：1协销，2会员机构，3供应商，4普通机构
     * @return List<ProductListVo>
     */
    @Override
    public ResponseJson<List<ProductListVo>> getMainProducts(Integer shopId, Integer identity) {
        if (shopId == null) {
            return ResponseJson.error("参数异常：缺少供应商Id", null);
        }
        // identity: 0个人，1协销，2会员机构，3供应商，4普通机构
        // p_visibility：3:所有人可见，2:普通机构可见，1:会员机构可见
        List<Integer> visibilityList = new ArrayList<>();
        if (identity == 1 || identity == 2) {
            // 协销 | 会员机构 | 综合供应商
            visibilityList.addAll(Arrays.asList(1, 2, 3));
        } else if (identity == 4) {
            // 普通机构
            visibilityList.addAll(Arrays.asList(2, 3));
        } else {
            // 游客|所有人
            visibilityList.add(3);
        }
        List<ProductListVo> list = shopMapper.getMainProducts(shopId, visibilityList);
        // 设置价格等级 及 老图片路径
        list.forEach(product -> {
            product.setPriceGrade(priceUtilService.getPriceGrade(product.getPrice()));
            product.setPrice(0d);
            product.setImage(ImageUtils.getImageURL("product", product.getImage(), 0, domain));
        });
        return ResponseJson.success(list);
    }

    /**
     * 供应商-我的商品列表
     *
     * @param shopId        供应商Id
     * @param name          商品名称
     * @param productCode   货号
     * @param validFlag     状态：0逻辑删除，1待审核，2已上架，3已下架，8审核未通过，9已隐身，10已冻结
     * @param featuredFlag  是否主推：0否，1是
     * @param commodityType 商品属性：1产品，2仪器
     * @param bigTypeId     一级分类Id
     * @param smallTypeId   二级分类Id
     * @param tinyTypeId    三级级分类Id
     * @param pageNum       页码
     * @param pageSize      每页数量
     * @return Map<String, Object>
     */
    @Override
    public ResponseJson<Map<String, Object>> getShopProducts(Integer shopId, String name, String productCode, Integer validFlag, Integer featuredFlag, Integer commodityType, Integer bigTypeId, Integer smallTypeId, Integer tinyTypeId, int pageNum, int pageSize) {
        if (null == shopId) {
            return ResponseJson.error("参数异常：缺少供应商Id", null);
        }
        Map<String, Object> map = new HashMap<>();
        PageHelper.startPage(pageNum, pageSize);
        List<ProductItemVo> productList = shopMapper.getShopProductsSelect(shopId, name, productCode, validFlag, featuredFlag, commodityType, bigTypeId, smallTypeId, tinyTypeId);
        if (null != productList && productList.size() > 0) {
            for (ProductItemVo product : productList) {
                String typeName = shopMapper.getTypeName(product.getBigTypeId(), product.getSmallTypeId(), product.getTinyTypeId());
                product.setTypeName(typeName);
                product.setIsChecked(false);
                // 设置商品主图及价格
                Integer userId = shopMapper.getUserIdByshopId(shopId);
                priceUtilService.setProductDetails(userId, product);
            }
        }
        PaginationVo<ProductItemVo> productPage = new PaginationVo<>(productList);
        map.put("productPage", productPage);
        int featuredNum = shopMapper.getMainProductsCount(shopId);
        map.put("featuredNum", featuredNum);
        return ResponseJson.success(map);
    }

    /**
     * 供应商-发布商品
     *
     * @param productDto {
     *                   aliasName	    内部商品名称
     *                   allAreaFlag	销售区域：1全部区域， 0指定区域
     *                   bigTypeId	    一级分类id
     *                   brandId	    品牌id
     *                   brandName	    品牌名称
     *                   commodityType	商品属性：1产品，2仪器
     *                   costPrice	    成本价
     *                   detailInfo	    商品详情信息
     *                   includedTax	是否含税：0不含税，1含税，2未知
     *                   invoiceType	发票类型
     *                   machineType	械字号类型
     *                   minBuyNumber	最小购买量
     *                   name	        商品名称
     *                   normalPrice	市场价
     *                   orderInfo	    订购方案
     *                   price	        机构价格
     *                   productCode	商品货号
     *                   productType	商品类型：0其它，1妆字号，2械字号
     *                   provinceIds	指定的销售区域
     *                   qualificationImg 械字号资质证书图
     *                   searchKey	      搜索关键词
     *                   serviceInfo	  服务详情
     *                   shopId	         供应商id
     *                   smallTypeId	 二级分类id
     *                   step	         购买数量
     *                   stock	         库存
     *                   tags	         商品标签，逗号隔开
     *                   taxPoint	     开票税点
     *                   tinyTypeId	     三级分类id
     *                   unit	         包装规格
     *                   }
     */
    @Override
    public ResponseJson releaseProduct(ProductDto productDto) {
        // 参数校验
        if (null == productDto.getShopId()) {
            return ResponseJson.error("参数异常：供应商id不能为空！", null);
        }
        if (StringUtils.isBlank(productDto.getName())) {
            return ResponseJson.error("参数异常：商品名称不能为空！", null);
        }
        if (StringUtils.isBlank(productDto.getAliasName())) {
            return ResponseJson.error("参数异常：内部商品名称不能为空！", null);
        }
        if (null == productDto.getCommodityType()) {
            return ResponseJson.error("参数异常：商品属性不能为空！", null);
        }
        if (null == productDto.getBigTypeId()) {
            return ResponseJson.error("参数异常：商品分类不能为空！", null);
        }
        if (null == productDto.getBrandId()) {
            return ResponseJson.error("参数异常：品牌id不能为空！", null);
        }
        if (null == productDto.getProductType()) {
            return ResponseJson.error("参数异常：商品类型不能为空！", null);
        }
        if (StringUtils.isBlank(productDto.getTags())) {
            return ResponseJson.error("参数异常：商品标签不能为空！", null);
        }
        if (StringUtils.isBlank(productDto.getUnit())) {
            return ResponseJson.error("参数异常：包装规格不能为空！", null);
        }
        if (null == productDto.getNormalPrice()) {
            return ResponseJson.error("参数异常：市场价不能为空！", null);
        }
        if (null == productDto.getPrice()) {
            return ResponseJson.error("参数异常：机构价格不能为空！", null);
        }
        if (null == productDto.getIncludedTax()) {
            return ResponseJson.error("参数异常：是否含税不能为空！", null);
        }
        if (null == productDto.getMinBuyNumber()) {
            return ResponseJson.error("参数异常：最小购买量不能为空！", null);
        }
        if (null == productDto.getStock()) {
            return ResponseJson.error("参数异常：库存不能为空！", null);
        }
        if (StringUtils.isBlank(productDto.getDetailInfo())) {
            return ResponseJson.error("参数异常：商品详情信息不能为空！", null);
        }
        JSONObject parseObject = JSON.parseObject(productDto.getParams());
        //图片
        List<Map<String, String>> imageList = (List<Map<String, String>>) parseObject.get("image");
        if (null == imageList || imageList.size() == 0) {
            return ResponseJson.error("请上传商品图片", null);
        }
        //相关参数
        List<Map<String, String>> paramList = (List<Map<String, String>>) parseObject.get("param");
        // 当前时间
        String current = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(new Date());
        //保存或修改商品
        ProductPo product = new ProductPo();
/*        if (null != productDto.getProductId()) {
            product.setProductId(productDto.getProductId());
        }
        product.setShopId(productDto.getShopId());
        product.setName(productDto.getName());
        product.setAliasName(productDto.getAliasName());
        product.setCommodityType(productDto.getCommodityType());
        product.setBigTypeId(productDto.getBigTypeId());
        product.setSmallTypeId(productDto.getSmallTypeId());
        product.setTinyTypeId(productDto.getTinyTypeId());
        product.setBrandId(productDto.getBrandId());
        product.setProductType(productDto.getProductType());
        product.setTags(productDto.getTags());
        product.setUnit(productDto.getUnit());
        product.setNormalPrice(productDto.getNormalPrice());
        product.setPrice(productDto.getPrice());
        product.setIncludedTax(productDto.getIncludedTax());
        product.setMinBuyNumber(productDto.getMinBuyNumber());
        product.setStock(productDto.getStock());
        product.setSearchKey(productDto.getSearchKey());
        product.setCostPrice(productDto.getCostPrice());
        product.setStep(productDto.getStep());
        product.setAllAreaFlag(productDto.getAllAreaFlag());
        product.setProvinceIds(productDto.getProvinceIds());
        product.setProductCode(productDto.getProductCode());
        product.setServiceNumber(productDto.getMinBuyNumber());
        product.setSupplierTaxPoint(productDto.getTaxPoint());
        product.setMachineType(productDto.getMachineType());
        product.setStep(productDto.getStep());
        product.setQualificationImg(productDto.getQualificationImg());
        product.setInvoiceType(productDto.getInvoiceType());
        product.setTaxPoint(productDto.getTaxPoint());
        product.setAllAreaFlag(productDto.getAllAreaFlag());
        product.setProvinceIds(productDto.getProvinceIds());*/
        BeanUtils.copyProperties(productDto,product);
        product.setUpdateTime(current);
        product.setValidFlag(1);
        product.setProductCategory(1);
        //设置主图
        for (Map<String, String> map : imageList) {
            String mainFlag = map.get("mainFlag");
            String image = map.get("image");
            if ("1".equals(mainFlag)) {
                product.setMainImage(image);
            }
        }
        if (null == productDto.getProductId()) {
            product.setAddTime(current);
            product.setHasSkuFlag(1);
            product.setSellNumber(0);
            product.setSortIndex(0);
            product.setFeaturedFlag(0);
            product.setCostCheckFlag(1);
            product.setRecommendType(0);
            product.setPriceFlag(2);
            product.setActFlag(0);
            product.setLadderPriceFlag(0);
            product.setVisibility(3);
            shopMapper.insertProduct(product);
            if (StringUtils.isBlank(productDto.getProductCode())) {
                String productCode = shopMapper.getCodeByTypeId(productDto.getBigTypeId(), productDto.getSmallTypeId(), productDto.getTinyTypeId());
                productCode = productCode + product.getProductId();
                product.setProductCode(productCode);
                shopMapper.updateProductCode(productCode, product.getProductId());
            }
        } else {
            shopMapper.updateProduct(product);
        }
        //保存或修改商品图片
        List<ProductImagePo> productImageList = shopMapper.getImageByProductId(product.getProductId());
        for (Map<String, String> map : imageList) {
            String mainFlag = map.get("mainFlag");
            String image = map.get("image");
            String productImageId = map.get("productImageId");
            ProductImagePo productImage = new ProductImagePo();
            productImage.setProductId(product.getProductId());
            productImage.setShopId(productDto.getShopId());
            productImage.setAddTime(current);
            productImage.setImage(image);
            productImage.setMainFlag(Integer.valueOf(mainFlag));
            if (StringUtils.isBlank(productImageId)) {
                shopMapper.insertProductImage(productImage);
            } else {
                productImage.setId(Integer.valueOf(productImageId));
                shopMapper.updateProductImage(productImage);
                productImageList.removeIf(s -> Integer.valueOf(productImageId).equals(s.getId()));
            }
        }
        if (null != productImageList && productImageList.size() > 0) {
            for (ProductImagePo imageItem : productImageList) {
                shopMapper.deleteProductImage(imageItem.getId());
            }
        }
        //保存相关参数
        shopMapper.deleteProductParameters(product.getProductId());
        if (null != paramList && paramList.size() > 0) {
            for (Map<String, String> map : paramList) {
                String paramsName = map.get("paramsName");
                String paramsContent = map.get("paramsContent");
                if (StringUtils.isNotBlank(paramsName) && StringUtils.isNotBlank(paramsContent)) {
                    ProductParameterPo parameters = new ProductParameterPo();
                    parameters.setParamsName(paramsName);
                    parameters.setParamsContent(paramsContent);
                    parameters.setProductId(product.getProductId());
                    parameters.setDelFlag(0);
                    shopMapper.insertProductParameters(parameters);
                }
            }
        }
        //保存修改商品详情
        ProductDetailInfoPo productDetail = new ProductDetailInfoPo();
        productDetail.setProductDetailInfoId(productDto.getProductDetailInfoId());
        productDetail.setProductId(product.getProductId());
        productDetail.setDetailInfo(productDto.getDetailInfo());
        productDetail.setServiceInfo(productDto.getServiceInfo());
        productDetail.setOrderInfo(productDto.getOrderInfo());
        if (null == productDetail.getProductDetailInfoId()) {
            shopMapper.insertProductDetailInfo(productDetail);
        } else {
            shopMapper.updateProductDetailInfo(productDetail);
        }
        return ResponseJson.success();
    }

    /**
     * 供应商-商品编辑回显
     *
     * @param productId 商品Id
     */
    @Override
    public ResponseJson<ProductFormVo> getProductForm(Integer productId) {
        if (null == productId) {
            return ResponseJson.error("参数异常：商品Id不能为空！", null);
        }
        ProductFormVo product = shopMapper.getProductForm(productId);
        if (null == product) {
            return ResponseJson.error("商品信息异常,productId:" + productId, null);
        }
        if (StringUtils.isNotBlank(product.getTags())) {
            String[] tags = product.getTags().split(",");
            product.setTagsList(tags);
        }
        String typeName = shopMapper.getTypeName(product.getBigTypeId(), product.getSmallTypeId(), product.getTinyTypeId());
        List<String> list = StringUtils.isNotEmpty(typeName) ? Arrays.asList(typeName.split("-")) : new ArrayList<>();
        product.setBigTypeName(list.size() > 0 ? list.get(0) : null);
        product.setSmallTypeName(list.size() > 1 ? list.get(1) : null);
        product.setTinyTypeName(list.size() > 2 ? list.get(2) : null);
        String brandName = shopMapper.getBrandNameById(product.getBrandId());
        product.setBrandName(brandName);
        ProductDetailInfoPo productDetail = shopMapper.getProductDetailInfo(productId);
        List<ProductImagePo> imageList = shopMapper.getProductImages(productId);
        for (ProductImagePo image : imageList) {
            String imageURL = ImageUtils.getImageURL("product", image.getImage(), 0, domain);
            image.setImage(imageURL);
        }
        List<ProductParameterPo> parametersList = shopMapper.getProductParameters(productId);
        product.setProductDetail(productDetail);
        product.setImageList(imageList);
        product.setParametersList(parametersList);
        return ResponseJson.success(product);
    }

    /**
     * 供应商-商品编辑-品牌联想
     *
     * @param userId 用户Id
     * @param name   关键词
     */
    @Override
    public ResponseJson<List<BrandVo>> brandAssociation(Integer userId, String name) {
        if (null == userId) {
            return ResponseJson.error("参数异常：用户id不能为空！", null);
        }
        if (StringUtils.isBlank(name)) {
            return ResponseJson.error("参数异常：联想关键词不能为空！", null);
        }
        List<BrandVo> bindList = shopMapper.getBrandAssociation(userId, name);
        if (null == bindList || bindList.size() == 0) {
            return ResponseJson.error("品牌名称暂未被采美收录", null);
        }
        return ResponseJson.success(bindList);
    }

    /**
     * 供应商-商品编辑-品牌检测
     *
     * @param name 品牌名称
     */
    @Override
    public ResponseJson<BrandVo> brandDetection(String name) {
        if (StringUtils.isBlank(name)) {
            return ResponseJson.error("参数异常：品牌名称不能为空！", null);
        }
        BrandVo brand = shopMapper.getBrandByName(name);
        if (null == brand) {
            return ResponseJson.error("品牌名称暂未被采美收录", null);
        }
        return ResponseJson.success(brand);
    }

    /**
     * 供应商-提交新品牌
     *
     * @param brandDto {
     *                 name        品牌名字
     *                 source      添加来源 0:后台添加，1:供应商添加
     *                 userId      添加用户ID,后台则为后台用户ID,供应商则为供应商用户ID
     *                 logo        品牌logo
     *                 description 品牌描述
     *                 }
     */
    @Override
    public ResponseJson<BrandPo> addBrand(BrandDto brandDto) {
        if (StringUtils.isBlank(brandDto.getName())) {
            return ResponseJson.error("参数异常：品牌名称不能为空！", null);
        }
        if (null == brandDto.getUserId()) {
            return ResponseJson.error("参数异常：用户id不能为空！", null);
        }
        BrandVo brandVo = shopMapper.getBrandByName(brandDto.getName());
        if (brandVo != null) {
            if (null == brandDto.getId()) {
                return ResponseJson.error("该品牌已存在，无需重复提交", null);
            } else if (!brandDto.getId().equals(brandVo.getId())) {
                return ResponseJson.error("该品牌已存在，无需重复提交", null);
            }
        }
        BrandPo brand = new BrandPo();
        brand.setId(brandDto.getId());
        brand.setName(brandDto.getName());
        brand.setUserId(brandDto.getUserId());
        brand.setSource(brandDto.getSource());
        brand.setLogo(brandDto.getLogo());
        brand.setDescription(brandDto.getDescription());
        brand.setStatus(0);
        brand.setDelFlag(0);
        Date createDate = new Date();
        brand.setUpdateDate(createDate);
        if (null == brand.getId()) {
            brand.setCreateDate(createDate);
            shopMapper.insertBrand(brand);
        } else {
            shopMapper.updateBrand(brand);
        }
        return ResponseJson.success(brand);
    }

    /**
     * 供应商-品牌管理
     *
     * @param name     品牌名称
     * @param status   品牌状态 0:待审核，1:审核通过，2:审核失败
     * @param userId   用户Id
     * @param pageNum  页码
     * @param pageSize 每页数量
     */
    @Override
    public ResponseJson<PaginationVo<BrandVo>> brandManagement(String name, Integer status, Integer userId, int pageNum, int pageSize) {
        if (null == userId) {
            return ResponseJson.error("参数异常：用户id不能为空！", null);
        }
        PageHelper.startPage(pageNum, pageSize);
        List<BrandVo> brandList = shopMapper.getShopBrandList(name, status, userId);
        PaginationVo<BrandVo> brandPage = new PaginationVo<>(brandList);
        return ResponseJson.success(brandPage);


    }

    /**
     * 供应商-添加,删除主页推荐
     *
     * @param featuredDto {
     *                    productId    商品Id
     *                    featuredFlag 供应商主推商品标志 0删除 1添加
     *                    shop         供应商Id
     *                    }
     */
    @Override
    public ResponseJson switchFeatured(FeaturedDto featuredDto) {
        if (null == featuredDto.getShopId()) {
            return ResponseJson.error("参数异常：供应商Id不能为空！");
        }
        if (null == featuredDto.getProductId()) {
            return ResponseJson.error("参数异常：商品Id不能为空！");
        }
        if (null == featuredDto.getFeaturedFlag()) {
            return ResponseJson.error("参数异常：供应商主推商品标志不能为空！");
        }
        if (1 == featuredDto.getFeaturedFlag()) {
            int featuredCount = shopMapper.getMainProductsCount(featuredDto.getShopId());
            if (featuredCount >= 4) {
                return ResponseJson.error("主推商品已经上限了！");
            }
        }
        int flag = shopMapper.updateProductFeatured(featuredDto.getProductId(), featuredDto.getShopId(), featuredDto.getFeaturedFlag());
        if (flag > 0) {
            return ResponseJson.success();
        } else {
            return ResponseJson.error("供应商主推商品标志更新失败！");
        }
    }

    /**
     * 供应商-批量下架商品
     *
     * @param productIds 商品id集合,以','隔开
     */
    @Override
    public ResponseJson soldOutProducts(String productIds) {
        if (StringUtils.isBlank(productIds)) {
            return ResponseJson.error("参数异常：请传入商品Id!");
        }
        String[] split = productIds.split(",");
        for (String productId : split) {
            if (StringUtils.isNotBlank(productId)) {
                ProductFormVo product = shopMapper.getProductForm(Integer.valueOf(productId));
                if (2 != product.getValidFlag()) {
                    return ResponseJson.error("只能下架已上架的商品");
                }
                PromotionsVo promotions = promotionsMapper.getPromotionsByProductId(product.getProductId());
                if (null != promotions) {
                    StringBuilder message = new StringBuilder();
                    if (promotions.getType() == 1) {
                        message = new StringBuilder("单品");
                    } else if (promotions.getType() == 2) {
                        message = new StringBuilder("凑单");
                    } else {
                        message = new StringBuilder("店铺");
                    }
                    if (promotions.getMode() == 1) {
                        message.append("优惠");
                    } else if (promotions.getMode() == 2) {
                        message.append("满减");
                    } else {
                        message.append("满赠");
                    }
                    return ResponseJson.error("此商品正在参与" + message + "促销活动，活动期间不能下架，如需强行下架，请联系客服：0755-22907771");
                }
                PromotionsVo promotionsGift = promotionsMapper.getPromotionGiftsByProductId(product.getProductId());
                if (null != promotionsGift) {
                    String message;
                    if (promotionsGift.getType() == 1) {
                        message = "单品满赠";
                    } else if (promotionsGift.getType() == 2) {
                        message = "凑单满赠";
                    } else {
                        message = "店铺满赠";
                    }
                    return ResponseJson.error("此商品是正在进行的" + message + "促销活动的赠品，活动期间不能下架，如需强行下架，请联系客服：0755-22907771");
                }
                product.setValidFlag(3);
                shopMapper.updateProductValidFlag(product.getProductId(), product.getValidFlag());
            }
        }
        return ResponseJson.success();
    }


}
